%{

/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


extern "C" {
  int yywrap() ;
  int yyparse() ;
}


#define EXTERN extern 
#define EXTERN_INIT(x,y) extern x

int newline(char*) ;

#ifdef hpux
#include <strings.h>
#else
#include <string.h>
#endif

#include "yoocom.hxx"
#include "yoocom_y.h"

static int first_time = 1 ;

%}

%s S_NORMAL S_SVCCLASS S_BODY S_CMT S_CMT_CLASS S_CMT_BODY 

VINT            [0-9][0-9]*
IDENTIFIER      [a-zA-Z_][a-zA-Z_0-9]*
ACCESS          public|private|protected
VIRTUAL         virtual
EQ_0            =[ \t]*0
REMOTE          remote
CHAR            char
U_CHAR          "unsigned char"
INT             int
U_INT           "unsigned int"
SHORT           short
U_SHORT         "unsigned short"
FLOAT           float
DOUBLE          double
STRING          "char*"
VOID            void
IN              IN
OUT             OUT
CONST           const
SVCCLASS        svc_class
SVCSTRUCT       svc_struct
FLOW            Flow|Flot
TABLE           Table
OBRACKET        \{
CBRACKET        \}
CXX_CMT         \/\/.*\n
C_BEGIN_CMT     \/\*
C_END_CMT       \*\/



%%

%{
  if (first_time == 1) {
    BEGIN S_NORMAL ;
    first_time = 0 ;
  }
%}

[ \t]           {
                  ECHO ;
		}

[\n]            {
                  ECHO ;
                  linecount++ ;
                }

{CXX_CMT}       {
                  ECHO ;
		  linecount++ ;
		}
<S_NORMAL>{C_BEGIN_CMT}   {
                  ECHO ;
		  BEGIN S_CMT ;
		}

<S_CMT>{C_END_CMT} {
                  ECHO ;
		  BEGIN S_NORMAL ;
		}

<S_CMT>.        {
                  ECHO ;
		}

<S_BODY>{C_BEGIN_CMT}   {
                  ECHO ;
		  BEGIN S_CMT_BODY ;
		}

<S_CMT_BODY>{C_END_CMT} {
                  ECHO ;
		  BEGIN S_BODY ;
		}

<S_CMT_BODY>.    {
                  ECHO ;
		}

<S_SVCCLASS>{C_BEGIN_CMT}   {
                  ECHO ;
		  BEGIN S_CMT_CLASS ;
		}

<S_CMT_CLASS>{C_END_CMT} {
                  ECHO ;
		  BEGIN S_SVCCLASS ;
		}

<S_CMT_CLASS>.    {
                  ECHO ;
		}

<S_NORMAL>{SVCCLASS}      {
                  fprintf(yyout, "class") ;
                  BEGIN S_SVCCLASS ;
		  return(SVCCLASS);
                }

<S_NORMAL>{SVCSTRUCT}     {
                  fprintf(yyout, "struct") ;
                  BEGIN S_SVCCLASS ;
		  return(SVCSTRUCT);
                }

<S_BODY>{OBRACKET}      {
                  ECHO; 
		  bracket_count++ ;
		}

<S_BODY>{CBRACKET}      {
                  strcpy(yylval.ident, (char*)yytext);		  
		  ECHO ;
		  if (--bracket_count == 1) {		    
		    BEGIN S_SVCCLASS ;
		  } 
		  return CBRACKET ;
		}

<S_SVCCLASS>{OBRACKET}      {
                  strcpy(yylval.ident, (char*)yytext);
		  if (bracket_count++ != 0) {
		    ECHO ;
		    BEGIN S_BODY ;
		  }
		  return OBRACKET ;
		}

<S_SVCCLASS>{CBRACKET}      {
                  bracket_count-- ;
		  if (bracket_count == 0) {
		    BEGIN S_NORMAL ;
		  }
                  return CBRACKET ;
		}


<S_SVCCLASS>{CHAR}          {
                  ECHO ;
                  strcpy(yylval.ident, (char*)yytext);
		  return(CHAR);
                }

<S_SVCCLASS>{U_CHAR}        {
                  ECHO ;
                  strcpy(yylval.ident, (char*)yytext);
		  return(U_CHAR);
                }

<S_SVCCLASS>{INT}           {
                  ECHO ;
                  strcpy(yylval.ident, (char*)yytext);
		  return(INT);
                }

<S_SVCCLASS>{U_INT}         {
                  ECHO ;
                  strcpy(yylval.ident, (char*)yytext);
		  return(U_INT);
                }

<S_SVCCLASS>{SHORT}         {
                  ECHO ;
                  strcpy(yylval.ident, (char*)yytext);
		  return(SHORT);
                }

<S_SVCCLASS>{U_SHORT}       {
                  ECHO ;
                  strcpy(yylval.ident, (char*)yytext);
		  return(U_SHORT);
                }

<S_SVCCLASS>{FLOAT}         {
                  ECHO ;
                  strcpy(yylval.ident, (char*)yytext);
		  return(FLOAT);
                }

<S_SVCCLASS>{DOUBLE}        {
                  ECHO ;
                  strcpy(yylval.ident, (char*)yytext);
		  return(DOUBLE);
                }

<S_SVCCLASS>{STRING}        {
                  ECHO ;
                  strcpy(yylval.ident, (char*)yytext);
		  return(STRING);
                }

<S_SVCCLASS>{VOID}          {
                  ECHO ;
                  strcpy(yylval.ident, (char*)yytext);
		  return(VOID);
                }

<S_SVCCLASS>{VINT}           {
		  yylval.value = atoi((char*)yytext);
		  return VINT;
                }


<S_SVCCLASS>{IN}            {
                  yylval.value = IN ;
		  return(IN);
                }

<S_SVCCLASS>{OUT}           {
                  yylval.value = OUT ;
		  return(OUT);
                }

<S_SVCCLASS>{CONST}           {
                  ECHO ;
		  return(CONST);
                }

<S_SVCCLASS>{FLOW}          {
                  ECHO ;
		  return(FLOW);
                }

<S_SVCCLASS>{TABLE}         {
                  ECHO ;
		  return(TABLE);
                }

<S_SVCCLASS>{REMOTE}         {
                  fprintf(yyout, "public") ;
		  return(REMOTE);
                }

<S_SVCCLASS>{ACCESS}         {
                  ECHO ;
		  return(ACCESS);
                }

<S_SVCCLASS>{VIRTUAL}         {
                  ECHO ;
		  return(VIRTUAL);
                }

<S_SVCCLASS>{EQ_0}         {
                  ECHO ;
		  return(EQ_0);
                }

<S_SVCCLASS>{IDENTIFIER}    {
                  ECHO ;
                  strcpy(yylval.ident, (char*)yytext);
		  return(IDENTIFIER);
                }

<S_SVCCLASS>.   { 
                  ECHO ;
                  return yytext[0];
		}


%%

int newline(char* str) {

  int cpt = 0 ;
  while ( (str = strrchr(str,'\n')) != NULL) {
    cpt++ ;
    str++ ;
  }
 
  return cpt;
}

